use utf8;
use Style;
use IPF;
{ IPF::run(\&tkx); } 1;
__END__

=head1 IPF

=for html
<a href="https://en.wikipedia.org/wiki/Iterative_proportional_fitting">Iterative Proportional Fitting</a><p>
Folgende Beispiel-Wanderungsmatrix
<table>
<tr><td>170</td><td>0</td></tr>
<tr><td>29,75</td><td>64,87</td></tr>
<tr><td>0</td><td>5,38</td></tr>
<tr><th>200</th><th>120</th><th>Untergrenze Spaltensumme</th></tr>
<tr><th>200</th><th>inf</th><th>Obergrenze Spaltensumme</th></tr>
</table>

kann als csv-Eingabedatei 2 verschiedene Formate haben:

"dense" als 2 dimensionale Matrix:

 # 1   ;  2        ;# Kommentare pro Zeile
 170   ;  0        ;# \
  29,75; 64,87     ;#  > 3 * 2 Matrix
   0   ; 55,38     ;# /
 200   ;120        ;# Untergrenze Spaltensumme
 200   ;inf        ;# Obergrenze Spaltensumme

"sparse" als 2 oder mehrdimensionale Matrix

 #von;nach;bg;gg;ag;Wert
    1;   1; 1; 1; 0; 170
    1;   2; 1; 1; 0;   0
    2;   1; 1; 1; 0;  29,75
    2;   2; 1; 1; 0;  64,87
    3;   1; 1; 1; 0;   0
    3;   2; 1; 1; 0;  55,38
    *;   1;200;200          ;# Unter-, Obergrenze Spaltensumme 1, oder Zielwert: *;1;200
    *;   2;120;inf          ;# Unter-, Obergrenze Spaltensumme 2, oder Zielwert: *;2;150
    1;   *; 1; 1; 0; 170    ;# optionales Zeilenziel 1
    2;   *; 1; 1; 0;  94,62 ;# optionales Zeilenziel 2
    3;   *; 1; 1; 0;  55,38 ;# optionales Zeilenziel 3

Voreinstellung für Zeilenziele sind die Zeilensummen der Ausgangsmatrix.
Bei Vorgabe von Zeilenzielen kann man für die Erreichung der Spaltenziele
eine neutrale "Saatmatrix", z.B. aus lauter Einsen vorgeben.

Die Summe der Zeilenziele muss (bis auf Gleichheitstoleranz) zwischen der Summe
der Unter- und Obergrenze Spaltensumme liegen.

Eine Obergrenze "inf" bedeutet "unendlich groß"

Die Ausgabedatei enthält die an die Spaltenziele und Zeilenziele
(Zeilensumme der Ausgangsmatrix) angepasste Matrix.

Die Iteration wird abgebrochen, wenn das Maximum der absoluten Differenz
der Spaltenabgleichsfaktoren zu 1 kleiner als die Abbruchschranke ist,
oder die Anzahl Iterationen die Maxiamle Anzahl Iterationen erreicht.

=head2 Parameter

=over

=item Eingabeformat

1 = "dense" oder 2 = "sparse" siehe oben

=item Maximale Anzahl Iterationen (iter)

Ist eps = 0, dann erfolgt der Abgleich mit exakt iter Iterationen,
sonst ist iter (z.B. 100, 200, 1000, ...) lediglich eine Obergrenze,
damit das Programm bei schlecht konditionierten Eingabedaten in keine
Endlosschleife gerät.

=item Abbruchschwelle (eps)

Die Iteration bricht ab, wenn beim Zeilenabgleich

 max(abs(1-f(1..r))) < eps

d.h. alle Zeilenkorrekturfaktoren fast genau 1 sind.

Sinnvolle Werte 1e-15, 5e-15, 1e-14, ... sind im Bereich der Genauigkeit
von 64-Bit Gleitkommazahlen.

=item Gleichheitstoleranz (q)

Wenn Summe Unter- (u) und/oder Summe Obergrenze (o) Spaltensumme gleich der
Matrixsumme (m) sein sollen, so erreicht man dies bei nicht ganzzahligen
Werten oft nicht exakt.

Deshalb erfolgt die Prüfung mit Gleicheitstoleranz q

 min(m/u-1, o/m-1) > -q

Für Q sind Werte von 1e-15, ..., 5e-15, ..., 1e-14, ... sinnvoll.

=item Protokollumfang

1: Sammelmeldung

 61 Iterationen, delta 6,66e-016, eps 1e-015, q 3e-15, verbose 1

2: zusätzlich
 Summe Matrix - Summe Untergrenze, Summe Obergrenze - Summe Matrix

 Iterationsprotokoll

  1 0,324657004154887
  2 0,109602391611596
  ...

3: umfangreiches Protokoll, nur für kleine Eingabedatei geeignet

=back

=head2 Eingabedatei

Bei einer SIKURS-Prognose mit Baustein I1/P1-4 kann man sich die Eingabedatei
für den Abgleich der Binnenwanderung und Innenwanderung mit

 Einwohner/Prognose/Berechen/Notiz
 !option($IPF n)
 n = 1: dense Format
 2 = 2: sparse Format

als Datei IPF2021.csv ausgeben lassen und optional mit

 Einwohner/Prognose/Berechen/Befehle/nach der Prognose
 IPF 'IPF2021', 'IPF2021_ipf', { format => 2, iter => 3, eps => 0, verbose => 1 };

als Datei IPF2021_ipf.csv abgleichen lassen

Alternativ Abgleich durch

 adeton.pl (ADETON Verfahren)
 .../IPF/IPF.htm (mit sce (Superkompensationsexponent))
