use utf8;
use Style;

my $directory = syscfg::get('DEFAULTS.Directory', '');
my $foundFile = tkx()->getOpenFile(
    -initialdir => ::OS::osPath($directory),
    -title => 'Eingabedatei für Blasendiagramm'
);
my @data;
my $pline = sub {
    push @data, shift;
    return 1;
};
if ($foundFile ne '' and CSV::iterate($foundFile, $pline)) {
    my $title  = shift @data;
    my $header = shift @data;
    my @tstring = ('['.join(',', map { "'$_'" } @$header).']');
    foreach (@data) {
        push @tstring, sprintf q(['%s', %s, %s, '%s', %s]), map { tr/,/./r } @$_;
    }
    # say dump(\@data, \@tstring);
    my ($fn, $dir, $ext) = fileparse($foundFile, qr/\.[^.]*/x);
    my $outFile = tkx()->getSaveFile(
        -initialdir => $dir,
        -initialfile => $dir.$fn.'.htm',
        -filetypes => [['Htm-Files', '.htm'], ['All Files', '*']],
        -defaultextension => '.htm',
        -title => "Ausgabedatei Blasendiagramm"
    );
    if ($outFile ne '' and open my $H, '>:utf8', $outFile) {
        printf {$H} << 'EOT', join(",\n", @tstring), $title->[0], $header->[1], $header->[2];
<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <title>Blasendiagramm</title>
    <script type="text/javascript" src="https://www.google.com/jsapi"></script>
    <script type="text/javascript">
      google.load("visualization", "1", {packages:["corechart"]});
      google.setOnLoadCallback(drawChart);
      function drawChart() {
        var data = google.visualization.arrayToDataTable([
%s
        ]);

        var options = {
          title: '%s',
          hAxis: {title: '%s'},
          vAxis: {title: '%s'},
          bubble: {textStyle: {fontSize: 11}}
        };

        var chart = new google.visualization.BubbleChart(document.getElementById('chart_div'));
        chart.draw(data, options);
      }
    </script>
  </head>
  <body>
    <div id="chart_div" style="width: 900px; height: 500px;"></div>
  </body>
</html>
EOT
        close $H;
        OS::browse($outFile);
    }
}
1;
__END__

=head1 bubble_chart

=for html
Experiment mit Google Chart Tool
<a href="https://google-developers.appspot.com/chart/interactive/docs/gallery/bubblechart">Bubble Chart</a> (Blasendiagramm)
<p>Der Benutzer erstellt eine CSV-Datei (z.B. mit Daten aus dem Tool Indikatoren)
<pre>
 "Korrelation Lebenserwartung mit Geburtenrate"
 ID ; "Lebenserwartung"; "Geburtenrate"; "Aggregat"; "Bevölkerung"
 101; 80,41            ; 1,48          ; Ost       ;  3728
 102; 81,59            ; 1,62          ; Ost       ;  4356
 103; 79,65            ; 1,53          ; Ost       ;  2873
 201; 76,44            ; 1,92          ; West      ;  1743
 202; 74,71            ; 2,31          ; West      ;  2165
</pre>
<table>
<tr><th>ID</th><td>Gebietskennung oder Gebietsname</td></tr>
<tr><th>Lebenserwartung</th><td>Attribut für die X-Achse</td></tr>
<tr><th>Geburtenrate</th><td>Attribut für die Y-Achse</td></tr>
<tr><th>Aggregat</th><td>Schlüssel oder Name</td></tr>
<tr><th>Bevölkerung</th><td>Attribut für die Blasengröße</td></tr>
</table>
