use utf8;
use Array::DeepUtils qw(value_by_path);
use List::MoreUtils  qw(pairwise);
use Style;

my $stz       = loc::get('record_separator');
my $directory = syscfg::get('DEFAULTS.Directory', '');
my $foundFile = tkx()->getOpenFile(
    -initialdir       => OS::osPath($directory),
    -title            => "Bitte wählen Sie eine Eingabedatei"
);
if ($foundFile ne '') {
    if (open(my $F, $foundFile)) {
        wprint($foundFile, ":\n");
        my ($fn, $dir) = fileparse($foundFile);
        my $outFile = tkx()->getSaveFile(
            -initialdir => $dir,
            -title      => "Bitte wählen Sie eine Ausgabedatei"
        );
        if ($outFile ne '') {
            my (@data, @cmnt);
            CSV::iterate($foundFile,
                sub {
                    my $fld = CSV::float(shift, \&Carp::croak);
                    wprint(join($stz, @$fld), "\n...\n") unless scalar @data;
                    push @data, $fld;
                    return 1;
                },
                sub {
                    my $l = shift;
                    wprint(join($stz, @$l), "\n");
                    push @cmnt, $l;
                    return 1;
                }
            );
            my @permute = (1 .. scalar(@{$data[0]}));
            my $permtxt = join(' ', @permute);
            my ($dlg, $bframe) = tkx()->dialogbox('dim_horizontal');
            $dlg->labelentry('Reihenfolge Spalten', -textvariable => \$permtxt, -width => 20)->pack(-anchor => 'w');
            $bframe->okbutton(
                sub {
                    @permute = split /\s+/, $permtxt;
                    my @ps = sort { $a <=> $b } @permute;
                    my @nn = (1 .. scalar(@ps));
                    my $ne = sum(pairwise { $a == $b } @ps, @nn);
                    if ((scalar(@permute) != scalar(@{$data[0]})) or ($ne != scalar(@ps))) {
                        wwarn("$permtxt ungültig ($ne)\n");
                        $dlg->g_destroy;
                        return;
                    }
                    for my $r (@cmnt) {
                        my @pr = @$r;
                        $pr[0] =~ s/^#//;
                        if (scalar(@pr) == scalar(@permute)) {
                            foreach my $i (0 .. $#pr) {
                                $r->[$i] = $pr[$permute[$i]-1];
                            }
                        }
                    }
                    for my $r (@data) {
                        my @pr =@$r;
                        foreach (0 .. $#$r) {
                            $r->[$_] = $pr[$permute[$_]-1];
                        }
                    }
                    my $O;
                    unless (open ($O, '>', $outFile)) {
                        wwarn("open $outFile $!\n");
                        $dlg->g_destroy;
                        return;
                    }
                    my (@min, @max, @dim);
                    my $a = [];
                    @min = @{$data[0]};
                    pop @min;
                    @max = @min;
                    foreach my $r (@data) {
                        $min[$_] = min($min[$_], $r->[$_]) foreach(0 .. scalar(@$r)-2);
                        $max[$_] = max($max[$_], $r->[$_]) foreach(0 .. scalar(@$r)-2);
                    }
                    $dim[$_] = $max[$_] - $min[$_] foreach (0 .. $#min);
                    foreach my $r (@data) {
                        my @idx;
                        $idx[$_] = $r->[$_] - $min[$_] foreach(0 .. $#min);
                        value_by_path($a, \@idx, $r->[-1]);
                    }
                    foreach my $c (@cmnt) {
                        print $O '#';
                        if (scalar (@$c) == (scalar(@min)+1)) {
                            my $v  = pop @$c;
                            my $ag = pop @$c;
                            CSV::print($O, [(map { '' } (0 .. scalar(@min)-2)), $ag]);
                            print $O '#';
                            CSV::print($O, [@$c, $min[-1] .. $max[-1]]);
                        } else {
                            CSV::print($O, $c);
                        }
                    }
                    my $walk;
                    $walk = sub {
                        my $x = shift;
                        if (defined $x) {
                            if (scalar(@_) < (scalar(@min)-1)) {
                                foreach my $ix (0 .. scalar(@$x)-1) {
                                    $walk->($x->[$ix], @_, $ix);
                                }
                            } else {
                                my @ixx;
                                $ixx[$_] = $_[$_] + $min[$_] foreach (0 .. $#min-1);
                                CSV::print($O, [@ixx, @$x]);
                            }
                        }
                    };
                    $walk->($a);
                    close $O;
                    wprint("> $outFile\n");
                    $dlg->g_destroy;
                },
                sub {
                    $dlg->g_destroy;
                    wprint("Abbruch\n");
                },
                whelp(),
            );
        }
    } else {
        wwarn("can not open $foundFile $!");
    }
}
1;
__END__

=head1 dim_horizontal

=for html
Lese strm-Datei oder beliebig andere Datei,
ändere optional die Reihenfolge der Spalten
und ordne die letzte
Index-Spalte in Ausgabedatei nebeneinander an.<br>
Dadurch reduziert sich die Anzahl Zeilen (bei ag um den Faktor 100) und man
kann sehr große strm-Dateien noch in EXCEL bearbeiten.<br>
Beispiel für sinnvolle Reihenfolge Spalten
<table>
<tr><td>1 2 3 4 5 6 7</td><td>Jahr von nach bg gg ag Wert</td></tr>
<tr><td>4 5 6 1 2 3 7</td><td>bg gg ag Jahr von nach Wert</td></tr>
</table>
(siehe auch ag_horizontal.pl)
