use utf8;
use Style;

my $jhr = 1900 + (localtime(time))[5]; # default Jahr
my ($r, $Z, $g, $s, $w) = (0, 0, 0, 0, 0);
my $initial = '0 Einwohner'; # z.B. '80,2 Mio Einwohner'
my $show    = 7;             # 1 + 2 + 4 (= Bits 0, 1, 2)
my $yrange  = '*:*';
my $size    = '600,400';
my $o = Math::ODE->new(
    verbose => 1,
    step    => 1,
    initial => [0], 
    ODE => [ sub { my ($t,$y) = @_; return $r*$y->[0] + $Z } ], 
    t0  => $jhr,
    tf  => $jhr + 20,
);
my ($dialog, $bframe) = tkx()->dialogbox('Globalprognose');
my $entry = sub {
    my $label  = shift;
    my $varref = shift;
    my $width  = shift // 10;
    $dialog->labelentry($label, -textvariable => $varref, -width => $width)->pack(-anchor => 'w');
};
$entry->('Startbevölkerung B0 (z.B. 82,5 Mio. Einwohner)', \$initial, 20);
$entry->('Geburtenziffer g (z.B. 9,54 Geburten pro 1000 Einwohner)', \$g);
$entry->('Sterbeziffer s (z.B. 11,04 Sterbefälle pro 1000 Einwohner)', \$s);
$entry->('Außenwegzugsziffer w (16.58 Außenwegzüge pro 1000 Einwohner)', \$w);
$entry->('Außenzuzug absolut Z (z.B. 1,85 Mio)', \$Z);
$entry->('Jahr Startbevölkerung t0', \$o->{t0});
$entry->('Endjahr Prognose tf', \$o->{tf});
$entry->('Ausgabe analytische Lösung (1) + Runge-Kutta (2) + Grenzwert (4)', \$show);
$entry->('Y-Achse (z.B. 0:* 20:100)', \$yrange);
$entry->('Größe', \$size);
my @b = $bframe->okbutton(
    sub {
        # Eingaben aufbereiten, mit +0 von string in Zahl wandeln
        my @ini = split/\s+/, $initial;   # 80,8 Mio. Einwohner
        my $B0  = (shift(@ini) // 0) + 0; # 80,8
        $o->{initial}[0] = $B0;
        my $ylabel = join(' ', @ini);   # Mio. Einwohner
        $Z += 0;
        $g += 0;
        $s += 0;
        $w += 0;
        # Gesamtrate r
        $r = ($g - $s - $w)/1000;
        $o->evolve();
        # Rückrechnen der Einzelbewegungen Z, G, S, W
        my $outfile = tkx()->getSaveFile(
            -initialfile      => 'bew',
            -title            => "Bitte wählen Sie eine Bewegungs-Ausgabedatei"
        );
        if (($outfile ne '') and open(my $CSV, '>', $outfile)) {
            wprint('> ', $outfile, "\n");
            my $valold = $B0;
            my $stz = loc::get('record_separator');
            print {$CSV} "# Globalprognose Bewegungen ($ylabel)\n";
            print {$CSV} '#', join($stz, qw(Jahr Anfbev. Zuzug Geburt Sterbefall Wegzug Endbev.)), "\n";
            foreach (sort { $a <=> $b } keys %{$o->{values}}) {
                my $val = $o->{values}{$_}[0];
                my $jhr = $_ + 0;
                my $delta = $val - $valold - $Z;
                my $r1000 = $g - $s -$w;         # Änderungsziffer pro 1000
                my $G = +($delta * $g / $r1000); # Geburten
                my $S = -($delta * $s / $r1000); # Sterbefälle
                my $W = -($delta * $w / $r1000); # Wegzüge
                print {$CSV} join($stz, $jhr, $valold, $Z, $G, $S, $W, $val), "\n";
                $valold = $val;
            }
            close $CSV;
        }
        # plotte analytische Lösung B(x), Näherungslösung, Grenzwert
        if (open my $G, gnuplot::cmdfilename('ODE')) {
            no locale;
            print {$G}
                '$data << EOD', "\n",
                join(' ', $o->{t0}, $B0), "\n",
                join("\n", map { join(' ', sprintf('%.15g', $_), sprintf('%.15g', @{$o->{values}{$_}})) } sort { $a <=> $b } keys %{$o->{values}}),
                "\nEOD\n" if $show & 2;
            my $Bs = ($r==0) ? '?' : -($Z/$r); # stabile Endbevölkerung
            my @plots;
            push @plots, q[B(x) title 'B(t)=(B_0+(Z/r)*e^{r*(t-t_0)}-(Z/r)'] if $show & 1;
            push @plots, q[$data title 'Runge-Kutta'] if $show & 2;
            push @plots, qq[$Bs title 'Bs'] if ($Bs ne '?') and ($show & 4);
            push @plots, q[0 notitle] if scalar(@plots) == 0; # leerer Plot
            print {$G} << "EOT";
B0 = $B0
r  = $r
Z  = $Z
x0 = $o->{t0}
B(x) = (B0+(Z/r)) * exp(r*(x-x0)) - (Z/r)
set title "B'=rB+Z B_0=$B0 g=$g s=$s w=$w r=(g-s-w)/1000=$r Z=$Z Bs=-(Z/r)=$Bs"
set term wxt 0 size $size enhanced
set ylabel '$ylabel'
set xrange[$o->{t0}:$o->{tf}]
set yrange[$yrange]
EOT
            print {$G} 'plot ', join(', ', @plots), "\n";
            print {$G} gnuplot::pause();
            wprint(gnuplot::cmdplot('ODE'));
        }
    },
    sub {
        $dialog->g_destroy;
        wprint("Abbruch\n");
    },
    whelp(),
);
Tkx::MainLoop();
1;
__END__

=head1 global_prognose

=for html
Experiment einer globalen Prognose
(d.h. ohne Differenzierung nach Alter, Geschlecht und Bevölkerungsgruppen)
mit Hilfe gewöhnlicher
<a href="http://sikurs.pthaug.de/maybe.htm#043">Differentialgleichungen</a>.<br>
Die Parameter entsprechen der Grundidee des SIKURS-Prognosebaukastens,
dass Geburten, Sterbefälle und Wegzüge als Raten und die
Zuzüge absolut vorgegeben werden:
<table>
<tr><th class="l">B<sub>0</sub></th><td>Ausgangsbevölkerung Jahres t<sub>0</sub> (*)</td></tr>
<tr><th class="l">g</th><td>Geburtenziffer - Anzahl Geburten pro 1000 Einwohner bezogen auf ein Jahr (*)</td></tr>
<tr><th class="l">s</th><td>Sterbeziffer - Anzahl Sterbefälle pro 1000 Einwohner bezogen auf ein Jahr (*)</td></tr>
<tr><th class="l">w</th><td>Au&szlig;enwegzugsziffer - Anzahl Au&szlig;enwegzüge pro 1000 Einwohner bezogen auf ein Jahr (*)</td></tr>
<tr><th class="l">Z</th><td>Au&szlig;enzuzug absolut pro Jahr (*)</td></tr>
<tr><th class="l">t<sub>0</sub></th><td>Jahresbezug Ausgangsbevölkerung B<sub>0</sub></td></tr>
<tr><th class="l">t<sub>f</sub></th><td>Endjahr der Prognose</td></tr>
</table>
(*) Werte (in Deutschland) mit Dezimalkomma vorgeben !
<dl>
<dt>Zuerst wird die Änderungsrate der Bevölkerung berechnet:</dt>
<dd>r = (g-s-w)/1000</dd>
<dt>Damit kann man die Differentialgleichung aufstellen:</dt>
<dd>B' = r*B + Z</dd>
<dt>mit der analytischen Lösung:</dt>
<dd>B(t) = (B<sub>0</sub>+Z/r)*e<sup>r*(t-t<sub>0</sub>)</sup>-Z/r</dd>
<dt>und dem stabilen Grenzwert der Bevölkerung</dt>
<dd>Bs = -Z/r bei r &lt; 0 für t&rarr;&infin; (bzw. r &gt; 0 für t&rarr;-&infin;)</dd>
</dl>
In der Plot-Ausgabe werden wahlweise
<ol>
<li value="1">der Funktionsgraph der analytischen Lösung B(t)</li>
<li value="2">die Werte einer Näherungslösung mit der Runge-Kutta Methode</li>
<li value="4">der stabile Grenzwert Bs</li>
</ol>
angezeigt.<br>
Die Skalierung der Y-Achse und die Größe der Graphik kann man vorgeben.<br>
Die Lesbarkeit der Graphik kann man verbessern, wenn man
B<sub>0</sub> und Z in einer geeigneten Einheit
(z.B. 80.8 für die Bevölkerung Deutschlands in Mio.) angibt.
<p>Die Einzelbewegungen pro Jahr werden aus der Differenz
End - Startbevölkerung rückgerechnet.
<ul>
<li>Der Zugug ist fix vorgegeben
<li>Geburten, Sterbefälle und Wegzüge werden im Verhältnis
g, s, w aufgeteilt
</ul>
Für eine globale Prognose mit Kapazitätsgrenze siehe
<a href="http://www.resnet.wm.edu/~jxshix/math410/Verhulst.html">Verhulst</a>
(/trunk/vc/verhulst.cpp)
