use utf8;
use Style;

if ((my $file = tkx()->getOpenFile(
    -title => "Bitte wählen Sie eine Datei mit Sterberaten",
    -initialdir => ::OS::osPath(syscfg::get('DEFAULTS.Directory', '')),
    -filetypes => tkx()->filetypes('csv', 'strb*.csv'),
)) ne '') {
    my ($fn, $dir) = fileparse($file);
    my @s; # Sterberaten
    my @mima;
    my $zeroes = 0;
    if (CSV::iterate(
            $file,
            sub {
                my @f = @{CSV::float(shift, \&Carp::croak)};
                $s[$f[0]-1][$f[1]-1][$f[2]-1][$f[3]] = $f[4];
                if ($f[4] == 0) {
                    $zeroes++;
                } else {
                    @mima = scalar(@mima) ? minmax(@mima, $f[4]) : ($f[4], $f[4]);
                }
                return 1;
            }
        )) {
        my @yrng = ((join(':', @mima) =~ tr/,/./r).' log', ['y-range', "0.0001:1 log\n0:0.5\n0:*\n*:*"]);
        my @x30 = (30, ['Untergrenze Darstellung Gompertz-Funktion', '30 Normalfall, 0, 5 oder 10 zum Vergleich mit unteren Altersgruppen']);
        my @ignore = (1, ['ignoriere oberste Altersgruppe', '0 nicht, 1 ja, da 99 und mehr']);
        my @parout = (1, ['Ausgabe gompertz-analyse.csv', '0 nicht, 1 ja, 2 anfügen']);
        my @fitlog = (0, ['Ausgabe fit.log', '0 nicht, 1 ja, 2 anfügen']);
        my @vb = qw(quiet results brief verbose);
        my $cnt = 0;
        my @verbose = (2, ['Protokollumfang', join(', ', map { $cnt++.' '.$_ } @vb)]);
        tkx('.t')->wprintf("< %s\n", $file);
        tkx('.t')->wprintf("[%s] Wertebereich Sterberaten ungleich 0\n", $yrng[0]);
        my ($dlg, $bfrm) = tkx()->dialogbox('Gompertz-Analyse');
        $dlg->labelentry($yrng[1], -textvariable => \$yrng[0], -width => 20)->pack(-anchor => 'w');
        $dlg->labelentry($x30[1], -textvariable => \$x30[0], -width => 5)->pack(-anchor => 'w');
        $dlg->labelentry($ignore[1], -textvariable => \$ignore[0], -width => 2, -vcmd => \&Tkxsik::valInt)->pack(-anchor => 'w');
        $dlg->labelentry($parout[1], -textvariable => \$parout[0], -width => 2, -vcmd => \&Tkxsik::valInt)->pack(-anchor => 'w');
        $dlg->labelentry($fitlog[1], -textvariable => \$fitlog[0], -width => 2, -vcmd => \&Tkxsik::valInt)->pack(-anchor => 'w');
        $dlg->labelentry($verbose[1], -textvariable => \$verbose[0], -width => 2, -vcmd => \&Tkxsik::valInt)->pack(-anchor => 'w');
        $bfrm->okbutton(
            sub {
                no locale;
                $dlg->g_destroy;
                my ($yrange, $logscale) = gnuplot::split_range($yrng[0], 'y', 10);
                my $parfn = $dir.'gompertz_analyse.csv';
                my $jahr = ($fn =~ m/(\d\d\d\d)/) ? $1 : 0; # Jahr aus strb2015.csv
                open my $G, gnuplot::cmdfilename('gompertz');
                print {$G} $logscale, "\n" if defined $logscale;
                printf {$G} <<EOT;
set key top left
set tics out
s(x)=S30*exp(G*(x-30))  # Gompertz-Funktion
EOT
                printf {$G} <<EOT, $parfn, $parout[0] == 2 ? ' append' : '' if $parout[0];
set print '%s'%s
EOT
                my $stz = CSV::STZ();
                printf {$G} <<EOT, join($stz, qw(Jahr TYS BG GG S30 G MRDT)) if $parout[0] == 1;
print '#%s'
EOT
                unlink $dir.'fit.log' if $fitlog[0] < 2;
                print {$G} 'set fit ', $fitlog[0] ? "logfile '".$dir."fit.log'" : 'nolog', ' ', $vb[$verbose[0]], "\n";
                my ($n, $undef_s30, $undef_s) = (0, 0, 0);
                my ($NTY, $NBG, $NGG, $NAG) = smart::dims(\@s);

                print {$G} '$STRB <<e', "\n";
                foreach my $ag (0 .. $NAG) {
                    print {$G} $ag;
                    foreach my $ty (0 .. $NTY-1) {
                        foreach my $bg (0 .. $NBG-1) {
                            foreach my $gg (0 .. $NGG-1) {
                                my $s_ = $s[$ty][$bg][$gg][$ag < $NAG ? $ag : $ag-1];
                                print {$G} ' ', defined($s_) ? sprintf('%.15g', $s_) : 'NaN';
                            }
                        }
                    }
                    print {$G} "\n";
                }
                print {$G} "e\n";

                foreach my $ty (0 .. $#s) {
                    foreach my $bg (0 .. $#{$s[$ty]}) {
                        foreach my $gg (0 .. $#{$s[$ty][$bg]}) {
                            ++$n;
                            my $G30 = $s[$ty][$bg][$gg][30] // 0;
                            if ($G30 == 0) {
                                my $msg = sprintf("%d %d %d 30 0: bei s(30)=0: G nicht definiert\n", $ty+1, $bg+1, $gg+1);
                                tkx('.t')->wwarn($msg);
                                print {$G} "#", $msg, "\n";
                                $undef_s30++;
                                next;
                            }
                            # fit ag 30-98 (da 99 und höher "immer" 0,5)
                            printf {$G} << 'EOT', $ty+1, $bg+1, $gg+1, sprintf('%.15g', $G30), gnuplot::wxtn($gnuplot::term{wxt}{set}, $n, $NTY*$NBG*$NGG), gnuplot::background(), $NAG, $#{$s[$ty][$bg][$gg]}-$ignore[0], $n+1;
# ty %d, bg %d, gg %d
S30 = %s
G = 0.08
set term %s %s
set xrange[0:%d]
fit [30:%d] s(x) $STRB using 1:%d via G
EOT
                            my $NAGplot = $NAG-$ignore[0]-1;
                            printf {$G} << 'EOT', $NAG, $yrange, $n+1, $fn, $ty+1, $bg+1, $gg+1, $x30[0], $NAGplot, $NAGplot;
set xrange[0:%d]
set yrange[%s]
set title sprintf('Gompertz-Diagramm s(30)=%%g G=%%g, MRDT=%%g', S30, G, log(2)/G)
set arrow 1 from 30.5, graph 0 to 30.5, graph 1 nohead lt 0
plot $STRB using 1:%d with steps title '%s %d %d %d', [%d.5:%d.5] s(x-0.5) title sprintf('s(30)*exp(G*(x-30)) [30:%d]')
EOT
                            printf {$G} <<EOT, join($stz, $jahr, $ty+1, $bg+1, $gg+1), $stz, $stz if $parout[0];
print "%s;",sprintf("%%.15g%s%%.15g%s%%.15g",S30,G,log(2)/G)
EOT
                        }
                    }
                }
                print {$G} gnuplot::pause();
                close($G);
                tkx('.t')->wprint(gnuplot::cmdplot('gompertz'));
                tkx('.t')->wprint($fitlog[0] == 1 ? '>' : '>>', " fit.log\n") if $fitlog[0];
                tkx('.t')->whint("$zeroes Sterberaten = 0\n") if $zeroes;
                tkx('.t')->whint("$undef_s Sterberaten nicht definiert\n") if $undef_s;
                tkx('.t')->wwarn("$undef_s30 undefinierte Sterberaten s(30)\n") if $undef_s30;
                tkx('.t')->wprint($parout[0] == 2 ? '>> ' : '> ', $parfn, "\n") if $parout[0];
            },
            sub {
                $dlg->g_destroy;
                wprint("Abbruch\n");
            },
            whelp(),
        );
        Tkx::MainLoop();
    } else {
        tkx('.t')->wwarn(CSV::errmsg($file), "\n");
    }
}
1;
__END__

=head1 gompertz

=for html
<a href="https://de.wikipedia.org/wiki/Mortalität">Modelierung Sterberate nach Gompertz</a><br>
<pre>s(a) = s(30)*exp(G*(a-30)) | 30 &le; a &lt; 99
mit
s(a) : Sterberate Alter a
G    : Gompertz Sterbekoeffizient (~0,08)
MRDT = log(2)/G: mortility rate doubling time (~7-9 Jahre)</pre>
Wenn in der Eingabedatei Altersgruppen fehlen, werden diese Altersgruppen
bei der Berechnung des Sterbekoeffizienten <code>G</code> nicht berücksichtigt,
d.h. die Berechnung liefert das gleiche Ergebnis, wie wenn die fehlende
Sterberate der Altergruppe den Wert der Gompertz Formel an dieser
Altergruppe hätte.
Wenn die Eingabedatei Sterberaten mit dem Wert 0 enthält, werden diese
bei der Berechnung berücksichtigt und reduzieren den Wert <code>G</code>.
Wurde die Sterberate der Eingabedatei als Quotient Sterbefälle/Bestand
berechnet, sollte bei 0 Sterbefällen:
bei 0 Bestand keine Sterberate, bei positivem Bestand Sterberate 0
vorgegeben werden.<br>
Fehlt in der Eingabedatei der Wert von Altersgruppe 30 oder ist s(30) = 0,
dann kann für diese demografische Gruppe kein <code>G</code> berechnet werden.
<p>Der Sterbekoeffizent <code>G</code> wird mittels Ausgleichsrechnung an die
Sterberaten der Altersgruppen 30 bis (99 -ignore) angeglichen.<br>
Sinnvolle Werte für ignore (ignoriere oberste Altersgruppe)
<ol>
<li value="0">Raten wurden für alle Altergruppen berechnet</li>
<li value="1">oberste Altersgruppe bedeutet "99 und mehr" und würde das Ergebnis verfälschen</li>
</ol>
<p>Mit den Werten <code>s(30)</code> und <code>G</code> können Sie die
Eingabedatei durch folgende idealisierte (geglättete) <code>strb</code>-Datei
ersetzen:
<pre>strb2017.csv:
#! expand(1)
#tys;bg;gg;ag;Rate
1;1;1;00;0,0123
1;1;1;01;0,00145
...
1;1;1;29;0,0001632
1;1;1;30..98;0,0001*exp(0,0872*($4-30))
1;1;1;99;0,4
...</pre>
mit <code>0,0001</code> sei <code>s(30)</code> (Sterberate der Altergruppe 30)
und <code>0,0872</code> sei <code>G</code> (Gompertz Sterbekoefizient)
(zur <code>MRDT 7,9401267</code>).<br>
<code>#! expand(1)</code> bewirkt, dass die Eingabedatei
<code>strb2017.csv</code> bei der Prognose in das Prognose-Unterverzeichnis
expandiert wird, d.h. alle berechneten Sterberaten der Altersgruppen
30 bis 98 enthält.
<p>Als Plausibilitätskontrolle kann man prüfen, ob
<ul>
<li>das Gompertz-Diagramm (mit logarithmischer Y-Achse) die charakteristische "Häkchen"-Form besitzt
<li><code>G ~ 0.08</code> ist
<li><code>MRDT</code> im Intervall 7-9 Jahre liegt
</ul>
<p>Mit dieser Methode könnte man eine Glättung und Extrapolation der
100 Parameter (s(0) .. s(99)) der Sterberaten durch Reduktion auf 8
Parameter durchführen:
<table>
<tr><th colspan="2">Alter</th><th>Parameter</th><th>Hinweis</th></tr>
<tr><td class="r">0 -</td><td class="r">9</td><td>s(a)</td><td>exponentiell fallend (fit a*exp(b*x)+c mit Startwert a=0.1 b=-1 c=0.1)<br>alterativ 10 Einzelwerte vorgeben</td></tr>
<tr><td class="r">10 -</td><td class="r">29</td><td>Mittelwert(s(i))</td><td>weitgehend konstant</td></tr>
<tr><td class="r">30 -</td><td cllass="r">98</td><td>s(30)*exp(G*(a-30))</td><td>Verdopplung alle MRDT Jahre</td></tr>
<tr><td colspan="2" class="r">99</td><td>s(a)</td><td>für 99 und älter (z.B. aus amtlicher Sterbetafel, oder m: 0,4, w: 0,3)</tr>
</table>
siehe auch Extras/Eigene Scripts/Start/weibull.pl
