use utf8;
use locale;
use List::Util qw(min max);
use Style;
use gnuplot;

my $directory = syscfg::get('DEFAULTS.Directory', '');
my $foundFile = tkx()->getOpenFile(
    -initialdir  => ::OS::osPath($directory),
    -defaultextension => 'strb*.csv',
    -title => "bitte wählen Sie eine strb-Datei (Sterberaten)"
);
return if $foundFile eq '';
wprint("< $foundFile\n");
my ($fn, $dir, $ext) = fileparse($foundFile, qr/\.[^.]*/x);
$fn =~ m/(\d\d\d\d)/;
my $of = tkx()->getSaveFile(
    -initialfile => 'le'.($1 // '').$ext,
    -initialdir  => $dir,
    -filetypes   => tkx()->csvfiles(),
    -title       => 'bitte wählen Sie eine Ausgabedatei'
);
my ($dialog, $bframe) = tkx()->dialogbox('Lebenserwartung');
my $logscale = 1;
$dialog->checkbutton('logarithmische Achse für Lebenserwartung', -variable => \$logscale)->pack(qw/-anchor w/);
$bframe->okbutton(
    sub {
        my $O;
        if ($of ne '') {
            if ($of eq $foundFile) {
                werror("Ausgabedatei $of gleich Eingabedatei\n");
                return;
            }
            unless (open($O, '>', $of)) {
                werror("open $of $!\n");
                $dialog->g_destroy;
                return;
            }
            wprint("> $of\n");
        }
        my @cmnt;
#       my (@min, @max, $i, $sum);
        my %strb;
        my $stz = CSV::STZ();
        my $cols;
        my $maxstrb = 0;
        my $minstrb = 1; # minimale Sterbesrate != 0
        my $n = 0;
        if (CSV::iterate(
            $foundFile,
            sub {
                my $fld = CSV::float(shift, \&Carp::croak);
                my $v = pop @$fld;
                $maxstrb = max($v, $maxstrb);
                $minstrb = min($v, $minstrb) if $v > 0;
                $cols = scalar(@$fld) unless defined($cols);
                my $ag = pop @$fld;
                $strb{join($stz, @$fld)}[$ag] = $v;
                return 1;
            },
            sub {
                my $fld = shift;
                my $n = scalar @$fld;
                if ($n > 2 and scalar(@cmnt) == 0) {
                    pop @$fld;
                    @cmnt = @$fld;
                }
                return 1;
            }
        )) {
            # berechnet Lebenserwartung (Anzahl Jahre bis Tod)
            # Andere Differenzierung als die Datei strb ist nicht sinnvoll/möglich
            open my $G, gnuplot::cmdfilename('le');
            wprint("maximale Sterberate ", $maxstrb, "\n");
            {
                no locale;
                printf {$G} <<e, ($logscale ? '' : '#'), $minstrb/2, $maxstrb;
set decimalsign locale 'german'
set ytics nomirror
set y2tics
%sset logscale y2
set y2range[%g:%g]
set y2label 'Sterbewahrscheinlichkeit'
set key left
set style data lines
e
            }
            unless (scalar @cmnt) {
                push @cmnt, "" foreach (1 .. $cols);
            }
            print {$O} join($stz, @cmnt, qw/q(x) p(x) l(x) d(x) L(x) T(x) e(q)/), "\n" if $of ne '';
            foreach my $k (sort keys %strb) {
                # Lebenserwartung ab 0 berechnen
                my (@p, @l, @d, @L, @T, @e);
                @p = map { 1 - ($_ // 0) } @{$strb{$k}};
                @l = (100000); # hier geht jede Zahl 1, 10, ..., 10000000
                $l[$_] = $p[$_-1] * $l[$_-1]              for (1 .. $#p);
                $d[$_] = $l[$_] - ($l[$_+1] // 0)         for (0 .. $#p);
                $L[$_] = 0.5 * ($l[$_] + ($l[$_+1] // 0)) for (0 .. $#p);
                $T[$_] = $L[$_] + ($T[$_+1] // 0) for reverse (0 .. $#p);
                $e[$_] = $T[$_] / $l[$_]                  for (0 .. $#p);
                if ($of ne '') {
                    print {$O} join($stz, $k, $_, ($strb{$k}[$_] // 0), $p[$_], $l[$_], $d[$_], $L[$_], $T[$_], $e[$_]), "\n" for (0 .. $#p);
                }
                print {$G} <<e;
\$DATA <<EOT
e
                my $lowerlimit = $logscale ? ($minstrb/10) : 0;
                for (0 .. $#p) {
                    my $s = max(($strb{$k}[$_] // 0), $lowerlimit);
                    print {$G} join(' ', $_, $s, $l[$_], $e[$_]), "\n";
                }
                printf {$G} <<e, ++$n, $k;
EOT
set term wxt %d enhanced size 800,600 title '%s'
plot\\
\$DATA using 1:(\$1+\$4) title 'Erwartungswert Sterbealter',\\
\$DATA using 1:4 title 'Lebenserwartung',\\
\$DATA using 1:2 axis x1y2 title 'Sterbewahrscheinlichkeit',\\
\$DATA using 1:(\$3/1000) title 'Überlebende * 1000'
e
            }
            print {$G} gnuplot::pause;
            close $G;
            gnuplot::cmdplot('le');
        } else {
            werror(CSV::errmsg(), "\n");
        }
        $dialog->g_destroy;
    },
    sub { $dialog->g_destroy; },
    whelp(),
);
Tkx::MainLoop();
1;
__END__

=head1 le_statis

=for html
Berechne differenzierte Lebenserwartung einer Datei mit Sterberaten
(siehe <a href="https://www.destatis.de/DE/Themen/Gesellschaft-Umwelt/Bevoelkerung/Sterbefaelle-Lebenserwartung/Publikationen/Downloads-Sterbefaelle/periodensterbetafel-erlaeuterung-5126203167004.pdf?__blob=publicationFile">STERBETAFEL 2014/2016</a>)<br>
Ergebnis weicht etwas vom vereinfachtgen Verfahren in lebenswerwartung.pl ab.
<p>Legende
<table>
<tr>
<th rowspan="3">Vollendetes Alter in Jahren</th>
<th>Sterbe-</th><th>Überlebens-</th>
<th rowspan="3">Überlebende im Alter x</th>
<th rowspan="3">Gestorbene im Alter x bis unter x+1</th>
<th colspan="2">Von den Überlebenden im Alter x</th>
<th rowspan="3">Durchschnittliche Lebenserwartung im Alter x in Jahren</th>
</tr>
<tr>
<th colspan="2" rowspan="2">wahrscheinlichkeit vom Alter x bis x+1</th>
<th>bis zum Alter x+1 durchlebte</th>
<th>insgesamt noch zu durchlebende</th>
<tr>
<th colspan="2">Jahre</th>
</tr>
<tr><th>x</th><th>q(x)</th><th>p(x)</th><th>l(x)</th><th>d(x)</th><th>L(x)</th><th>T(x)</th><th>e(x)</th></tr>
</table>
