use utf8;
use Style;

# folgende Variablen bitte Ihren Wünschen anpassen:
my $NAGmax = '099'; # Höchste Altersgruppe

my $directory = syscfg::get('RATEN.DIRDSTBEST', '');
wprint("mikrosim.pl: dstbest -> gem + reftyp\n");
my $foundFile = tkx()->getOpenFile(
    -initialdir  => ::OS::osPath($directory),
    -filetypes   => tkx()->filetypes('txt'),
    -initialfile => 'dstbest.txt',
    -defaultextension => '.txt',
    -title => "Bitte wählen Sie einen Statiskdatensatz Bestand dstbest"
);
if ($foundFile ne '') {
    my ($fn, $dir) = fileparse($foundFile);
    syscfg::put('RATEN.DIRDSTBEST', $dir);
    syscfg::flush(sub { wprint(@_, "\n") });
    my $sig = '7';
    return 1 if tkx()->askBox( "mikrosim",
        'ok',
        "Anzahl signifikate Stellen R02 (0-7)",
        -textvariable => \$sig,
        -width => 5
    ) ne 'ok';
    my $plus = 4 + 7 - $sig; # Ergänzng mit laufender Nummer
    my $fmt; # Ausgabeformat Gebietskennzeichen GEM
    if ($sig > 0) {
        $fmt = "%0${sig}d%0${plus}d%04d";
    } else {
        $fmt = "%s%0${plus}d%04d";
    }
    my $R02sOld = '';
    my $R03old  = '';
    my %R02plus;
    my %reftyp;
    my $jahr;    # aus Z02 für GEM
    open(my $D, '<', $foundFile) or die "open $foundFile $!";
    wprint('< ', $foundFile, "\n");
    my $savefile = sub {
        my $name = shift;
        my $path = tkx()->getSaveFile(
            -initialfile => $name,
            -initialdir => ::OS::osPath($directory),
            -title => "Bitte wählen Sie eine Ausgabedatei"
        );
        die "Abbruch Wahl $name" if $path eq '';
        return $path;
    };
    my $reftyp = $savefile->('reftyp.csv');
    open(my $REF, '>', $reftyp) or die "open $reftyp $!";
    wprint('> ', $reftyp, "\n");
    my $GEM;
    printf {$REF} "#TGKZ;Name;G;S,B;Z;Q;N;R;W;A;H;A\n#!%d\n", 15-$sig;
    my $show = 3; # Anzahl Eingabesätze anlisten
    while (<$D>) {
        chomp;
        my ($R01, $R03, $RN1, $R02) = unpack 'a8a11a4a7', $_;
        my $R02s = substr($R02, 0, $sig); # signifikante Stellen von R02
        my $P02  = substr($_, 116, 1);    # Geschlecht
        my $P09  = substr($_, 117, 2);    # Art der deutschen Staatsangeörigkeit
        my $HHNR = substr($_, 182, 4);    # Nummer des Haushalts an der Basisadresse
        my $A01  = substr($_, 250, 3);    # Alter der Person
        unless (defined $jahr) {
            $jahr = substr($_, 226, 4);
            my $gem = $savefile->("gem${jahr}.csv");
            open($GEM, '>', $gem) or die "open $gem $!";
            print {$GEM} "#Jahr;GKZ,BG;GG;AG;Anzahl\n";
            wprint('> ', "${dir}GEM${jahr}_.csv\n");
        }
        wprint("R01:$R01,R03:$R03,RN1:$RN1,R02:$R02,P02:$P02,P09:$P09,HHNR:$HHNR,A01:$A01\n") if $. <= $show;
        wprint("...\n") if $. == $show;
        if (($R02s ne $R02sOld) or ($R03 ne $R03old)) {
            $R02plus{$R02s}++;
        }
        my $haushalt = sprintf $fmt, $R02s, $R02plus{$R02s}, $HHNR;
        print {$GEM} join(';', $jahr, $haushalt, $P09 == 0 ? 1 : 2, $P02, min($A01, $NAGmax), 1), "\n";
        $reftyp{$R02s}++;
        $R02sOld = $R02s;
        $R03old  = $R03;
    }
    wprintf("Saetze: < %s: %d, > REFTYP: %d, > GEM: %d\n", $fn, $., scalar(keys %reftyp), $.);
    close $D;
    close $GEM;
    foreach (sort keys %reftyp) {
        print {$REF} join(';', $sig ? $_ : 1, '', 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1), "\n";
    }
    close $REF;
}
1;
__END__

=head1 mikrosim

Beispiel für die Ableitung einer GEM-Datei und REFTYP-Datei aus einem
Statistikdatensatz dstbest für eine sehr kleinräumige Prognose auf
Basis Haushalte durch eine ganzzahlige stochastische nicht deterministische
Mikrosimulation mit der Monte-Carlo Methode.

Wählen Sie im Programm die Anzahl signifikanter Stellen s zwischen 0-7.
Dies bestimmt den Umfang der REFTYP-Datei.
Bei s = 0 besteht die REFTYP-Datei aus nur einem Eintrag, d.h. mit
nur einem Typ Geburtenraten, Sterberaten, etc.

Beachte:
In der GEM-Datei treten z.B. bei Zwilligen 2 Sätze mit gleichem
Schlüssel auf.
Das SIKURS-Einleseprogramm addiert diese zu einem Matrixelement mit
Anzahl 2

Für eine Prognose brauchen Sie dann noch die je nach
Bausteinkombination Raten, wie z.B. fruc und strb.

Bei der Prognose wählen Sie dann in der Startmaske

 [1 0  ] Monte-Carlo Methode

Nach der Prognose können Sie mit Ergebnis/Zeitreihe/Start

 Aggregation über
 [substr(0,-13)  ] Gebietskennzeichen

die Ergebnissse auf die ersten 3 Stellen von R02 hochaggregieren
