use utf8;
use locale;
use Style;
use Data::Dump qw(dump);
use Data::Float qw(float_is_nan float_is_infinite);
use POSIX qw( strtod );

my $directory = syscfg::get('DEFAULTS.Directory', '');
my $foundFile = tkx()->getOpenFile(
    -initialdir  => ::OS::osPath($directory),
    -title => "please select input file"
);
if ($foundFile ne '') {
    my ($cmnts, $recs, $empty) = (0, 0, 0);
    my ($v, $n_unparsed, @min, @max, $i, $sum);
    my ($fn, $dir) = fileparse($foundFile);
    my $outFile = tkx()->getSaveFile(
        -initialdir => $dir,
        -title => "please select output file"
    );
    if ($outFile ne '') {
        my $stz = CSV::STZ();
        die "can not overwrite\n" if $foundFile eq $outFile;
        open (my $O, '>', $outFile) or die "open $outFile $!";
        my $undefs = 0;
        my $infins = 0;
        my $agold = -1;
        my $vold  = 0;
        my $pline = sub {
            my $fld = shift;
            $recs++;
            my $vs = pop @$fld;
            ($v, $n_unparsed) = strtod($vs);
            my $ag = pop @$fld;
            if ($ag < $agold) {
                $vold = 0;
            }
            if (float_is_nan($v)) {
                $v = $vold;
                $undefs++;
            }
            if (float_is_infinite($v)) {
                $v = $vold;
                $infins++;
            }
            push @$fld, $ag, $v;
            print {$O} join($stz, @$fld), "\n";
            if ($ag > $agold) {
                $vold = $v unless float_is_nan($v) or float_is_infinite($v);
            }
            $agold = $ag;
            return 1;
        };
        my $pcomment = sub {
            my $line = join($stz, @{$_[0]})."\n";
            print {$O} $line;
            wprint($line);
            return 1;
        };
        wprint("< $foundFile:\n");
        if (CSV::iterate($foundFile, $pline, $pcomment)) {
            # o.k.
        } else {
            wprint("$foundFile $!\n");
        }
        close $O;
        wprint("...\n");
        wprint("> $outFile\n");
        wprintf("%d Saetze mit %d ersetzten 'undef', %d ersetzten 'Inf'\n", $recs, $undefs, $infins);
    } else {
        wprint("can not open $foundFile $!");
    }
}
1;
__END__

=head1 undef

Ersetze 'nan' und 'inf' mit besetztem Wert in niedriger Altersgruppe.

Dateien mit 'nan' und 'inf' entstehen bei 'SIKURS Eingabedaten berechnen'

bei Option

 [1 ] undefinierte Werte

'nan': sowohl Zähler (Bewegung) als auch Nenner (Bestand) Null oder undefiniert

'inf': Zähler (Bewegung) ungleich Null und Nenner (Bestand) Null oder undefiniert
