use utf8;
use locale;
use Style;

# Anpassbare Parameter
my @og = (9, 29, 98); # Altersobergrenze 1 Baby, 2 jung, 3 alt (nicht 99 und älter)
my %weibull = (
    T1 => -10,
    b1 =>   4,
    t1 =>  $og[0]+1,
    c2 =>   0.0004,
    T3 => 100,
    b3 =>   4,
    t3 =>  $og[1]+1,
    fit_t1_t3 => 0,     # Anpassung ausfallfreie Zeit ?
    parout => 1,
    fitlog => 0,
); # Startwert Weibull-Parameter für Anpassung
my $extra_xrange = 0;   # 0-10 zur Beurteilung untere/obere Weibull-Funktion
my $yrange = '-0.01:*'; # '0:0.5' '0:1' '0:*' '*:*' '0.0001:1 logscale y 10'
my $yrange_txt = ['y-range', "0:0.5\n0:1\n0:*\n*:*\n0.0001:1 log"];
my $showWpar = 1;

my $directory = syscfg::get('DEFAULTS.Directory', '');
my $file = tkx()->getOpenFile(
    -title => "please select strb or weibull_synthese file",
    -initialdir  => ::OS::osPath($directory),
);
if ($file ne '') {
    my ($fn, $dir) = fileparse($file);
    if ($fn =~ m/^strb/i) {
        my @mima;
        my $zeroes = 0;
        my @s;
        if (CSV::iterate(
                $file,
                sub {
                    my @f = @{CSV::float(shift, \&Carp::croak)};
                    $s[$f[0]-1][$f[1]-1][$f[2]-1][$f[3]] = $f[4];
                    if ($f[4] == 0) {
                        $zeroes++;
                    } else {
                        @mima = scalar(@mima) ? minmax(@mima, $f[4]) : ($f[4], $f[4]);
                    }
                    return 1;
                }
            )) {
            my $yrngc;
            {
                no locale;
                $yrngc = join(':', @mima);
            }
            tkx('.t')->wprintf("< %s\n", $file);
            tkx('.t')->whintf("%d Sterberaten mit Wert 0\n", $zeroes) if $zeroes;
            tkx('.t')->wprintf("[%s] Wertebereich Sterberaten ungleich 0\n", $yrngc);
            my $ogs = join(' ', @og);
            my ($dlg, $bframe) = tkx()->dialogbox('Weibull-Analyse');
            $dlg->labelentry('Altersobergrenzen jung mittel alt', -textvariable => \$ogs, -width => 20)->pack(-anchor => 'w');
            $dlg->labelentry($yrange_txt, -textvariable => \$yrange, -width => 20)->pack(-anchor => 'w');
            $dlg->checkbutton('Parameter t1, t3 anpassen?', -variable => \$weibull{fit_t1_t3})->pack(-anchor => 'w');
            $dlg->labelentry(['Ausgabe weibull-analyse.csv', '0 nicht, 1 ja, 2 anfügen'], -textvariable => \$weibull{parout}, -width => 2, -vcmd => \&Tkxsik::valInt)->pack(-anchor => 'w');
            $dlg->labelentry(['Ausgabe fit.log', '0 nicht, 1 ja, 2 anfügen'], -textvariable => \$weibull{fitlog}, -width => 2, -vcmd => \&Tkxsik::valInt)->pack(-anchor => 'w');
            my @b = $bframe->okbutton(
                sub {
                    #$dlg->g_destroy;
                    @og = split / /, $ogs;
                    # Voreinstellung ausfallfreie Zeiten
                    $weibull{t1} = $og[0]+1;
                    $weibull{t3} = $og[1]+1;
                    weibull::analyse(tkx('.t'), \@s, \@og, \%weibull, $yrange, $dir, $fn);
                },
                sub {
                    $dlg->g_destroy;
                    wprint("Abbruch\n");
                },
                whelp(),
            );
        }
        Tkx::MainLoop();
    } elsif ($fn =~ m/^weibull_synthese/i) {
        my $setStrbUndef0 = 0;
        my ($dlg, $bframe) = tkx()->dialogbox('Weibull-Synthese');
        $dlg->labelentry(['Erweiterung Altersbereich', "0-10 zur Beurteilung untere/obere Weibull-Fumktion"], -textvariable => \$extra_xrange, -width => 20)->pack(-anchor => 'w');
        $dlg->labelentry($yrange_txt, -textvariable => \$yrange, -width => 20)->pack(-anchor => 'w');
        $dlg->labelentry(['Ausgabeformat Weibull-Funktion in strb-Date', "1 als normal strb-Datei mit Rate pro Altersgruppe\n2 zusätzlich mit Ausgabe Weibull-Parameter als Kommentar\n3 Ausgabe als Funkion pro demografische Gruppe"], -textvariable => \$showWpar, -width => 5)->pack(-anchor => 'w');
        $dlg->checkbutton('Ersetze in strb Undefinierte Werte durch 0', -variable => \$setStrbUndef0)->pack(-anchor => 'w');
        my @b = $bframe->okbutton(
            sub {
                $dlg->g_destroy;
                weibull::synthese(tkx('.t'), $dir, $fn, $extra_xrange, $yrange, $setStrbUndef0, $showWpar);
            },
            sub {
                $dlg->g_destroy;
                wprint("Abbruch\n");
            },
            whelp(),
        );
        Tkx::MainLoop();
    } else {
        werror("Name ungültig: $file\n");
    }
}
1;
__END__

=head1 weibull

=for html
Experiment mit <a href="http://www.weibull.de/WeibullHTML.htm">Weibullverteilung</a>
w(t)=1-exp(-(((t-t0)/(T-t0))^b))<br>
mit (in Worten der Zuverlässigkeitsanalyse)<br>
w(t) = Ausfallwahrscheinlichkeit bzw. Ausfallhäufigkeit (normiert auf 1, in % mal 100)<br>
t = Lebensdauervariable (Altersgruppe, Fahrstrecke, Einsatzdauer, Lastwechsel usw.)<br>
T = Charakteristische Lebensdauer, bei der 63.2% der Einheiten ausgefallen sind
    (für t=T gilt H = 100% (1 - 1/e) = 63.2 % ).<br>
b = Formparameter, Steigung der Ausgleichsgeraden im Weibull-Netz<br>
t0 = ausfallfreie Zeit
<p>Der Name der ausgewählten Datei bestimmt die Betriebsart Analyse
oder Synthese
<ul>
<li>Weibull-Analyse<br>
Wähle eine strb-Datei und berechne die Badewannenkurve der Sterberaten
mit 3 Weibullverteilungen
für früh, mittel, spät und "99 und älter".<br>
Fehlende Altersgruppen in der strb-Datei werden als unbekannte Sterberaten
gewertet, enthält die strb-Datei Sterberaten 0 für "unbekannte Sterberate",
sollten diese in der strb-Datei auskommentiert oder gelöscht werden.<br>
Die optimalen Altersgrenzen für die Abgrenzung der 3 Altersklassen
ist noch unklar, deshalb kann der Anwender diese vorgeben.<br>
Weiterhin kann der Anwender vorgeben, ob der Parameter t0,
dessen Voreinstellung sich aus der Abgrenzung der Altersklassen ergibt,
angepasst werden soll.<br>
Die Parameter der Weibullverteilungen werden aus den
Stützpunkten der strb-Datei mittles 
"nonlinear least-squares Marquardt-Levenberg algorithm" nebenbei bei der
Visualisierung durch das gnuplot-Programm berechnet.<br>
Liefert schlechte Ergebnisse bei unrealistischen Sterberaten, wie z.B.
<pre>96;0,25 # realistischer Wert
97;0,01 # zu gering, da aus zu kleiner Stichprobe berechnet
98;0    # Stichprobe ohne Sterbefall für Altersgruppe
99;1    # manuell zu hoch gesetzt</pre>
Ausgabe der 11 Weibull-Parameter pn=0..10 pro demographischr Gruppe in
<pre>weibull_analyse.csv:
#Jahr;TYS;BG;GG;A1;A2;A3;T1;b1;t1;c2;T3;b3;t3;s(99)
1992;1;1;1;9;29;99;-10,0;4,0;10;0,001;100,0;4,0;30;0,4
1992;1;1;2;9;29;99;-10,0;4,0;10;0,001;110,0;4,0;30;0,3
...
</pre>
Wiederholt man die Weibull-Analyse mit mehreren strb2010.csv, strb2011.csv, ...,
dann enthält <code>weibull_analyse.csv</code> bei der Option
"Datei anfügen" die Weibull-Parameter aller strb-Dateien.
<li>Weibull-Synthese<br>
Die Fortschreibung der Weibull-Parameter in die Zukunft erfolgt manuell,
durch Erstellung und Auswahl einer Datei
<code>weibull_synthese.csv</code><br>
Der Benutzer kann dazu versuchen Weibull-Parameter
aus <code>weibull_analyse.csv</code> z.B. durch
lineare Regression in die Zukunft zu extrapolieren und zusätzlich Annahmen
(Autor ist med. Laie) für die Entwicklung der Sterberaten in den Lebensphasen
<ol>
    <li>fruh: bessere Geburtsbegleitung, ...
    <li>mittel: autonome Autos reduziert Anzahl Verkehrstote,
    Suchtprgramme für Alkohol und Drogen greifen, ...
    <li>spät: medizinischer Durchbruch bei Krebs, Alzheimer, etc.
    verlängert Lebenserwartung,
    Fettleibigkeit und Bewegungsmangel reduziert diese, ...
</ol>
durch Alternativrechnungen zu testen.<br>
Bei der Synthese wird <code>weibull_synthese.csv</code> gelesen und
<ul>
<li>in eine Folge von strb-Dateien umgewandelt<br>
Die strb-Dateien enthalten wahlweise pro Altersgruppe die mit der
Weibull-Funktion berechnete Sterberate, oder die Weibull-Funktion für
eine Klasse von Altersjahren:
<pre>strb2019.csv:
#tys;bg;gg;ag    ;Weibull  t    T     b  t0
   1; 1; 1; 0.. 4;weibull($4 (-235) 1,35 5)  # fruh: fallende Kurve
   1; 1; 1; 5..29;0,001                      # mittel: konstant
   1; 1; 1;30..98;weibull($4 104 5 30)       # spät: steigende Kurve
   1; 1; 1;    99;0,4
...</pre>
<li>visualisiert
</ul>
Wenn die Weibullparameter ungünstig sind, können bei der Synthese bei
einzelnen Altersgruppen undefinierte Werte "1.#QNAN" entstehen, wenn graphisch
interpretiert die Sterberate unter Null sinken würde.<br>
In der strb-Ausgabedatei wird dann optional der Ersatzwert 0 eingesetzt,
in der Visualisierung wird die Linie unterbrochen.
</ul>
</ul>
<p>Bei der Visualisierung der Sterberaten ist es (wie bei jeder
Exponentialkurve) hilfreich eine
<a href="https://de.wikipedia.org/wiki/Logarithmische_Darstellung">logarithmische Skalierung</a>
für y-range zu
wählen, z.B
<pre>0.00001:1 log
1e-5:1 log
2**-7:1 logscale y 2</pre>
(siehe auch <a href="gnuplot.htm"><code>set logscale</code></a>)
<p>Mehr zur Modellierung von Sterbewahrscheinlichkeiten siehe
<a href="http://www.fam.tuwien.ac.at/~sgerhold/pub_files/theses/marguerite.pdf"> Mehodik von Sterblichkeitsuntersuchungen</a>
oder
<a href="http://www.math.tugraz.at/~aistleitner/Proseminar20072008/Krenn_Handout.pdf">Ausgleich von Sterbetafeln</a>
<p>Auswertungen mit logarithmischer Sklaierung zeigen, dass das Verfahren nicht
optimal ist, deshalb siehe<br>
Extras/Eigene Scripts/Start/gomperts.pl (mit besseren Ergebnissen für Altersbereich 30 .. 98)
