use utf8;
use Style;
use File::Basename;
use Data::Dump qw(dump);
use CSV;
use loc;

my $directory = syscfg::get('DEFAULTS.Directory', '');
my $version   = syscfg::get('DEFAULTS.Version', '');
my $K = (vercfg::get('METHODE.AUSSENWANDERUNGSDATEN', 'K0') =~ tr/K//dr);
die "Baustein K$K ungültig\n" unless 1 <= $K and $K <= 4;
my %M;
foreach (qw(NTYA NBG NGG NAG)) {
    $M{$_} = vercfg::get('METHODE.'.$_, 0);
}
my $stz = CSV::STZ();
wprint("Normierung zudq für Baustein K$K (NTYA=$M{NTYA}, NBG=$M{NBG}, NGG=$M{NGG}, NAG=$M{NAG})\n");
my $foundFile = tkx()->getOpenFile(
    -initialdir  => ::OS::osPath($directory),
    -filetypes   => [['zudq-Files', 'zudq*.csv'], ['All Files', '*.csv']],
    -defaultextension => '.csv',
    -title => "please select input file"
);
if ($foundFile ne '') {
    my ($cmnts, $recs, $empty) = (0, 0, 0);
    my ($f, $d, $e) = fileparse($foundFile, qr/\.[^.]*/x);
    my $outFile = tkx()->getSaveFile(
        -initialdir  => $d,
        -initialfile => $f.'_K'.$K.'_norm.csv',
        -title => "please select output file"
    );
    if ($outFile ne '') {
        die "can not overwrite\n" if $foundFile eq $outFile;
        open (my $O, '>:encoding(UTF-8)', $outFile) or die "open $outFile $!";
        my @zudq;
        my @tysu;
        my $pline = sub {
            my $fld = CSV::float(shift, \&Carp::croak);
            $recs++;
            wprint(join($stz, @$fld), "\n") if $recs <= 3;
            my $v = $fld->[4] + 0;
            $zudq[$fld->[0]-1][$fld->[1]-1][$fld->[2]-1][$fld->[3]] = $v;
            if ($K == 1) {
                $tysu[0] += $v;
            } elsif ($K == 2) {
                $tysu[$fld->[0]-1] += $v;
            } elsif ($K == 3) {
                $tysu[$fld->[0]-1][$fld->[1]-1] += $v;
            } elsif ($K == 4) {
                $tysu[$fld->[0]-1][$fld->[1]-1][$fld->[2]-1] += $v;
            }
            return 1;
        };
        my $pcmnt = sub {
            print {$O} (join($stz, @{$_[0]}), "\n");
            return 1;
        };
        wprint("< $foundFile:\n");
        CSV::iterate($foundFile, $pline, $pcmnt) or die "error $foundFile $!\n";
        wprint("...\n");
        my @default = (
            0,
            $M{NTYA}*$M{NBG}*$M{NGG}*$M{NAG},
            $M{NBG}*$M{NGG}*$M{NAG},
            $M{NBG}*$M{NAG},
            $M{NAG}
        );
        my $def = 1/$default[$K];
        my @minmax = (1e30, -1e30);
        my $ersatz = sprintf("Ersatzwert %g wegen fehledender Vorgaben", $def);
        wwarnf("%s\n", $ersatz) if $K == 1 and not defined $tysu[0];
        foreach my $tya (0 .. $M{NTYA}-1) {
            wwarnf("%s für TYA %d\n", $ersatz, $tya+1) if $K == 2 and not defined $tysu[$tya];
            foreach my $bg (0 .. $M{NBG}-1) {
                wwarnf("%s für TYA %d, BG %d\n", $ersatz, $tya+1, $bg+1) if $K == 3 and not defined $tysu[$tya][$bg];
                foreach my $gg (0 .. $M{NGG}-1) {
                    wwarnf("%s für TYA %d, BG %d, GG %d\n", $ersatz, $tya+1, $bg+1, $gg+1) if $K == 4 and not defined $tysu[$tya][$bg][$gg];
                    foreach my $ag (0 .. $M{NAG}-1) {
                        my $nenner;
                        if ($K == 1) {
                            $nenner = $tysu[0] // 0;
                        } elsif ($K == 2) {
                            $nenner = $tysu[$tya] // 0;
                        } elsif ($K == 3) {
                            $nenner = $tysu[$tya][$bg] // 0;
                        } elsif ($K == 4) {
                            $nenner = $tysu[$tya][$bg][$gg] // 0;
                        }
                        $minmax[0] = $nenner if $nenner < $minmax[0];
                        $minmax[1] = $nenner if $nenner > $minmax[1];
                        my $wert = ($nenner == 0) ? $def : ($zudq[$tya][$bg][$gg][$ag] // 0) / $nenner;
                        print {$O} join($stz, $tya+1, $bg+1, $gg+1, $ag, $wert), "\n";
                    }
                }
            }
        }

        close $O;
        wprint("> $outFile\n");
        wprintf("%d Sätze mit Typsummen im Bereich %f .. %f skaliert\n", $recs, @minmax);
    } else {
        wprint("can not open $foundFile $!");
    }
}
1;
__END__

=head1 zudq_norm

Normiere eine zudq Datei, so dass je nach Baustein K die Summen

 K1: über TYA, BG, GG, AG
 K2: je TYA über BG, GG, AG
 K3: je TYA, BG über GG, AG
 K4: je TYA, BG, GG über AG

gleich 1 sind
